@@ -1,4 +1,31 @@
+# MIT License
+#
+# Copyright (c) 2016-2022 Mark Qvist / unsigned.io
+#
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in all
+# copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
+
 from .vendor.platformutils import get_platform
+from cryptography.hazmat.primitives import hashes
+from cryptography.hazmat.primitives.kdf.hkdf import HKDF
+from cryptography.hazmat.backends import default_backend
+
+cio_default_backend = default_backend()
 
 if get_platform() == "android":
     from .Interfaces import Interface
@@ -6,6 +33,7 @@ if get_platform() == "android":
     from .Interfaces import AutoInterface
     from .Interfaces import TCPInterface
     from .Interfaces import UDPInterface
+    from .Interfaces import I2PInterface
 else:
     from .Interfaces import *
 
@@ -48,12 +76,12 @@ class Reticulum:
     other programs to use on demand.
     """
 
-    # Future minimum will probably be locked in at 244 bytes to support
-    # networks with segments of different MTUs. Absolute minimum is 211.
+    # Future minimum will probably be locked in at 251 bytes to support
+    # networks with segments of different MTUs. Absolute minimum is 219.
     MTU            = 500
     """
     The MTU that Reticulum adheres to, and will expect other peers to
-    adhere to. By default, the MTU is 500 bytes. In custom RNS network
+    adhere to. By default, the MTU is 507 bytes. In custom RNS network
     implementations, it is possible to change this value, but doing so will
     completely break compatibility with all other RNS networks. An identical
     MTU is a prerequisite for peers to communicate in the same network.
@@ -62,6 +90,29 @@ class Reticulum:
     the default value.
     """
 
+    MAX_QUEUED_ANNOUNCES = 16384
+    QUEUED_ANNOUNCE_LIFE = 60*60*24
+
+    ANNOUNCE_CAP = 2
+    """
+    The maximum percentage of interface bandwidth that, at any given time,
+    may be used to propagate announces. If an announce was scheduled for
+    broadcasting on an interface, but doing so would exceed the allowed
+    bandwidth allocation, the announce will be queued for transmission
+    when there is bandwidth available.
+
+    Reticulum will always prioritise propagating announces with fewer
+    hops, ensuring that distant, large networks with many peers on fast
+    links don't overwhelm the capacity of smaller networks on slower
+    mediums. If an announce remains queued for an extended amount of time,
+    it will eventually be dropped.
+
+    This value will be applied by default to all created interfaces,
+    but it can be configured individually on a per-interface basis.
+    """
+
+    MINIMUM_BITRATE = 500
+
     # TODO: To reach the 300bps level without unreasonably impacting
     # performance on faster links, we need a mechanism for setting
     # this value more intelligently. One option could be inferring it
@@ -75,8 +126,10 @@ class Reticulum:
 
     HEADER_MINSIZE = 2+1+(TRUNCATED_HASHLENGTH//8)*1
     HEADER_MAXSIZE = 2+1+(TRUNCATED_HASHLENGTH//8)*2
+    IFAC_MIN_SIZE  = 1
+    IFAC_SALT      = bytes.fromhex("adf54d882c9a9b80771eb4995d702d4a3e733391b2a0f53f416d9f907e55cff8")
     
-    MDU            = MTU - HEADER_MAXSIZE
+    MDU            = MTU - HEADER_MAXSIZE - IFAC_MIN_SIZE
 
     router         = None
     config         = None
@@ -139,6 +192,8 @@ class Reticulum:
         self.share_instance       = True
         self.rpc_listener         = None
 
+        self.ifac_salt = Reticulum.IFAC_SALT
+
         self.requested_loglevel = loglevel
         if self.requested_loglevel != None:
             if self.requested_loglevel > RNS.LOG_EXTREME:
@@ -273,13 +328,71 @@ class Reticulum:
         self.__start_local_interface()
 
         if self.is_shared_instance or self.is_standalone_instance:
+            RNS.log("Bringing up system interfaces...", RNS.LOG_DEBUG)
             interface_names = []
             for name in self.config["interfaces"]:
                 if not name in interface_names:
+                    # TODO: We really need to generalise this way of instantiating
+                    # and configuring interfaces. Ideally, interfaces should just
+                    # have a conrfig dict passed to their init method, and return
+                    # a ready interface, onto which this routine can configure any
+                    # generic or extra parameters.
+
                     c = self.config["interfaces"][name]
 
+                    interface_mode = Interface.Interface.MODE_FULL
+                    
+                    if "interface_mode" in c:
+                        if c["interface_mode"] == "full":
+                            interface_mode = Interface.Interface.MODE_FULL
+                        elif c["interface_mode"] == "access_point" or c["interface_mode"] == "accesspoint" or c["interface_mode"] == "ap":
+                            interface_mode = Interface.Interface.MODE_ACCESS_POINT
+                        elif c["interface_mode"] == "pointtopoint" or c["interface_mode"] == "ptp":
+                            interface_mode = Interface.Interface.MODE_POINT_TO_POINT
+
+                    elif "mode" in c:
+                        if c["mode"] == "full":
+                            interface_mode = Interface.Interface.MODE_FULL
+                        elif c["mode"] == "access_point" or c["mode"] == "accesspoint" or c["mode"] == "ap":
+                            interface_mode = Interface.Interface.MODE_ACCESS_POINT
+                        elif c["mode"] == "pointtopoint" or c["mode"] == "ptp":
+                            interface_mode = Interface.Interface.MODE_POINT_TO_POINT
+
+                    ifac_size = None
+                    if "ifac_size" in c:
+                        if c.as_int("ifac_size") >= Reticulum.IFAC_MIN_SIZE*8:
+                            ifac_size = c.as_int("ifac_size")//8
+                            
+                    ifac_netname = None
+                    if "networkname" in c:
+                        if c["networkname"] != "":
+                            ifac_netname = c["networkname"]
+                    if "network_name" in c:
+                        if c["network_name"] != "":
+                            ifac_netname = c["network_name"]
+
+                    ifac_netkey = None
+                    if "passphrase" in c:
+                        if c["passphrase"] != "":
+                            ifac_netkey = c["passphrase"]
+                    if "pass_phrase" in c:
+                        if c["pass_phrase"] != "":
+                            ifac_netkey = c["pass_phrase"]
+                            
+                    configured_bitrate = None
+                    if "bitrate" in c:
+                        if c.as_int("bitrate") >= Reticulum.MINIMUM_BITRATE:
+                            configured_bitrate = c.as_int("bitrate")
+                            
+                    announce_cap = Reticulum.ANNOUNCE_CAP/100.0
+                    if "announce_cap" in c:
+                        if c.as_float("announce_cap") > 0 and c.as_float("announce_cap") <= 100:
+                            announce_cap = c.as_float("announce_cap")/100.0
+                            
                     try:
-                        if ("interface_enabled" in c) and c.as_bool("interface_enabled") == True:
+                        interface = None
+
+                        if (("interface_enabled" in c) and c.as_bool("interface_enabled") == True) or (("enabled" in c) and c.as_bool("enabled") == True):
                             if c["type"] == "AutoInterface":
                                 if not RNS.vendor.platformutils.is_windows():
                                     group_id        = c["group_id"] if "group_id" in c else None
@@ -300,18 +413,26 @@ class Reticulum:
                                         ignored_interfaces
                                     )
 
-                                    if "outgoing" in c and c.as_bool("outgoing") == True:
-                                        interface.OUT = True
-                                    else:
+                                    if "outgoing" in c and c.as_bool("outgoing") == False:
                                         interface.OUT = False
+                                    else:
+                                        interface.OUT = True
+
+                                    interface.mode = interface_mode
+
+                                    interface.announce_cap = announce_cap
+                                    if configured_bitrate:
+                                        interface.bitrate = configured_bitrate
+                                    if ifac_size != None:
+                                        interface.ifac_size = ifac_size
+                                    else:
+                                        interface.ifac_size = 16
 
-                                    RNS.Transport.interfaces.append(interface)
                                 else:
                                     RNS.log("AutoInterface is not currently supported on Windows, disabling interface.", RNS.LOG_ERROR);
                                     RNS.log("Please remove this AutoInterface instance from your configuration file.", RNS.LOG_ERROR);
                                     RNS.log("You will have to manually configure other interfaces for connectivity.", RNS.LOG_ERROR);
 
-
                             if c["type"] == "UDPInterface":
                                 device       = c["device"] if "device" in c else None
                                 port         = int(c["port"]) if "port" in c else None
@@ -336,19 +457,27 @@ class Reticulum:
                                     forward_port
                                 )
 
-                                if "outgoing" in c and c.as_bool("outgoing") == True:
-                                    interface.OUT = True
-                                else:
+                                if "outgoing" in c and c.as_bool("outgoing") == False:
                                     interface.OUT = False
+                                else:
+                                    interface.OUT = True
 
-                                RNS.Transport.interfaces.append(interface)
+                                interface.mode = interface_mode
 
+                                interface.announce_cap = announce_cap
+                                if configured_bitrate:
+                                    interface.bitrate = configured_bitrate
+                                if ifac_size != None:
+                                    interface.ifac_size = ifac_size
+                                else:
+                                    interface.ifac_size = 16
 
                             if c["type"] == "TCPServerInterface":
                                 device       = c["device"] if "device" in c else None
                                 port         = int(c["port"]) if "port" in c else None
                                 listen_ip    = c["listen_ip"] if "listen_ip" in c else None
                                 listen_port  = int(c["listen_port"]) if "listen_port" in c else None
+                                i2p_tunneled = c.as_bool("i2p_tunneled") if "i2p_tunneled" in c else False
 
                                 if port != None:
                                     listen_port = port
@@ -358,37 +487,94 @@ class Reticulum:
                                     name,
                                     device,
                                     listen_ip,
-                                    listen_port
+                                    listen_port,
+                                    i2p_tunneled
                                 )
 
-                                if "outgoing" in c and c.as_bool("outgoing") == True:
-                                    interface.OUT = True
-                                else:
+                                if "outgoing" in c and c.as_bool("outgoing") == False:
                                     interface.OUT = False
+                                else:
+                                    interface.OUT = True
 
-                                RNS.Transport.interfaces.append(interface)
+                                if interface_mode != Interface.Interface.MODE_FULL:
+                                    RNS.log(str(interface)+" does not support Access Point mode, reverting to default mode: Full", RNS.LOG_WARNING)
+                                    interface_mode = Interface.Interface.MODE_FULL
+                                
+                                interface.mode = interface_mode
 
+                                interface.announce_cap = announce_cap
+                                if configured_bitrate:
+                                    interface.bitrate = configured_bitrate
+                                if ifac_size != None:
+                                    interface.ifac_size = ifac_size
+                                else:
+                                    interface.ifac_size = 16
 
                             if c["type"] == "TCPClientInterface":
                                 kiss_framing = False
                                 if "kiss_framing" in c and c.as_bool("kiss_framing") == True:
                                     kiss_framing = True
+                                i2p_tunneled = c.as_bool("i2p_tunneled") if "i2p_tunneled" in c else False
+
 
                                 interface = TCPInterface.TCPClientInterface(
                                     RNS.Transport,
                                     name,
                                     c["target_host"],
                                     int(c["target_port"]),
-                                    kiss_framing = kiss_framing
+                                    kiss_framing = kiss_framing,
+                                    i2p_tunneled = i2p_tunneled
                                 )
 
-                                if "outgoing" in c and c.as_bool("outgoing") == True:
-                                    interface.OUT = True
-                                else:
+                                if "outgoing" in c and c.as_bool("outgoing") == False:
                                     interface.OUT = False
+                                else:
+                                    interface.OUT = True
 
-                                RNS.Transport.interfaces.append(interface)
+                                if interface_mode != Interface.Interface.MODE_FULL:
+                                    RNS.log(str(interface)+" does not support Access Point mode, reverting to default mode: Full", RNS.LOG_WARNING)
+                                    interface_mode = Interface.Interface.MODE_FULL
+                                
+                                interface.mode = interface_mode
 
+                                interface.announce_cap = announce_cap
+                                if configured_bitrate:
+                                    interface.bitrate = configured_bitrate
+                                if ifac_size != None:
+                                    interface.ifac_size = ifac_size
+                                else:
+                                    interface.ifac_size = 16
+
+                            if c["type"] == "I2PInterface":
+                                i2p_peers = c.as_list("peers") if "peers" in c else None
+                                connectable = c.as_bool("connectable") if "connectable" in c else False
+
+                                interface = I2PInterface.I2PInterface(
+                                    RNS.Transport,
+                                    name,
+                                    Reticulum.storagepath,
+                                    i2p_peers,
+                                    connectable = connectable,
+                                )
+
+                                if "outgoing" in c and c.as_bool("outgoing") == False:
+                                    interface.OUT = False
+                                else:
+                                    interface.OUT = True
+
+                                if interface_mode != Interface.Interface.MODE_FULL:
+                                    RNS.log(str(interface)+" does not support Access Point mode, reverting to default mode: Full", RNS.LOG_WARNING)
+                                    interface_mode = Interface.Interface.MODE_FULL
+                                
+                                interface.mode = interface_mode
+
+                                interface.announce_cap = announce_cap
+                                if configured_bitrate:
+                                    interface.bitrate = configured_bitrate
+                                if ifac_size != None:
+                                    interface.ifac_size = ifac_size
+                                else:
+                                    interface.ifac_size = 16
 
                             if c["type"] == "SerialInterface":
                                 port = c["port"] if "port" in c else None
@@ -410,12 +596,20 @@ class Reticulum:
                                     stopbits
                                 )
 
-                                if "outgoing" in c and c.as_bool("outgoing") == True:
-                                    interface.OUT = True
-                                else:
+                                if "outgoing" in c and c.as_bool("outgoing") == False:
                                     interface.OUT = False
+                                else:
+                                    interface.OUT = True
 
-                                RNS.Transport.interfaces.append(interface)
+                                interface.mode = interface_mode
+
+                                interface.announce_cap = announce_cap
+                                if configured_bitrate:
+                                    interface.bitrate = configured_bitrate
+                                if ifac_size != None:
+                                    interface.ifac_size = ifac_size
+                                else:
+                                    interface.ifac_size = 8
 
                             if c["type"] == "KISSInterface":
                                 preamble = int(c["preamble"]) if "preamble" in c else None
@@ -451,12 +645,20 @@ class Reticulum:
                                     beacon_data
                                 )
 
-                                if "outgoing" in c and c.as_bool("outgoing") == True:
-                                    interface.OUT = True
-                                else:
+                                if "outgoing" in c and c.as_bool("outgoing") == False:
                                     interface.OUT = False
+                                else:
+                                    interface.OUT = True
 
-                                RNS.Transport.interfaces.append(interface)
+                                interface.mode = interface_mode
+
+                                interface.announce_cap = announce_cap
+                                if configured_bitrate:
+                                    interface.bitrate = configured_bitrate
+                                if ifac_size != None:
+                                    interface.ifac_size = ifac_size
+                                else:
+                                    interface.ifac_size = 8
 
                             if c["type"] == "AX25KISSInterface":
                                 preamble = int(c["preamble"]) if "preamble" in c else None
@@ -493,12 +695,20 @@ class Reticulum:
                                     flow_control
                                 )
 
-                                if "outgoing" in c and c.as_bool("outgoing") == True:
-                                    interface.OUT = True
-                                else:
+                                if "outgoing" in c and c.as_bool("outgoing") == False:
                                     interface.OUT = False
+                                else:
+                                    interface.OUT = True
 
-                                RNS.Transport.interfaces.append(interface)
+                                interface.mode = interface_mode
+
+                                interface.announce_cap = announce_cap
+                                if configured_bitrate:
+                                    interface.bitrate = configured_bitrate
+                                if ifac_size != None:
+                                    interface.ifac_size = ifac_size
+                                else:
+                                    interface.ifac_size = 8
 
                             if c["type"] == "RNodeInterface":
                                 frequency = int(c["frequency"]) if "frequency" in c else None
@@ -529,24 +739,61 @@ class Reticulum:
                                     id_callsign = id_callsign
                                 )
 
-                                if "outgoing" in c and c.as_bool("outgoing") == True:
-                                    interface.OUT = True
-                                else:
+                                if "outgoing" in c and c.as_bool("outgoing") == False:
                                     interface.OUT = False
+                                else:
+                                    interface.OUT = True
+
+                                interface.mode = interface_mode
+
+                                interface.announce_cap = announce_cap
+                                if configured_bitrate:
+                                    interface.bitrate = configured_bitrate
+                                if ifac_size != None:
+                                    interface.ifac_size = ifac_size
+                                else:
+                                    interface.ifac_size = 8
+
+                            if interface != None:
+                                interface.ifac_netname = ifac_netname
+                                interface.ifac_netkey = ifac_netkey
+
+                                if interface.ifac_netname != None or interface.ifac_netkey != None:
+                                    ifac_origin = b""
+
+                                    if interface.ifac_netname != None:
+                                        ifac_origin += RNS.Identity.full_hash(interface.ifac_netname.encode("utf-8"))
+
+                                    if interface.ifac_netkey != None:
+                                        ifac_origin += RNS.Identity.full_hash(interface.ifac_netkey.encode("utf-8"))
+
+                                    ifac_origin_hash = RNS.Identity.full_hash(ifac_origin)
+                                    interface.ifac_key = HKDF(
+                                        algorithm=hashes.SHA256(),
+                                        length=64,
+                                        salt=self.ifac_salt,
+                                        info=None,
+                                        backend=cio_default_backend,
+                                    ).derive(ifac_origin_hash)
+
+                                    interface.ifac_identity = RNS.Identity.from_bytes(interface.ifac_key)
+                                    interface.ifac_signature = interface.ifac_identity.sign(RNS.Identity.full_hash(interface.ifac_key))
 
                                 RNS.Transport.interfaces.append(interface)
+
                         else:
                             RNS.log("Skipping disabled interface \""+name+"\"", RNS.LOG_DEBUG)
 
                     except Exception as e:
                         RNS.log("The interface \""+name+"\" could not be created. Check your configuration file for errors!", RNS.LOG_ERROR)
                         RNS.log("The contained exception was: "+str(e), RNS.LOG_ERROR)
-                        # TODO: Remove
-                        raise e
                         RNS.panic()
                 else:
                     RNS.log("The interface name \""+name+"\" was already used. Check your configuration file for errors!", RNS.LOG_ERROR)
                     RNS.panic()
+
+            RNS.log("System interfaces are ready", RNS.LOG_DEBUG)
+
                 
 
     def __create_default_config(self):
@@ -569,6 +816,9 @@ class Reticulum:
                     if path == "interface_stats":
                         rpc_connection.send(self.get_interface_stats())
 
+                    if path == "path_table":
+                        rpc_connection.send(self.get_path_table())
+
                     if path == "next_hop_if_name":
                         rpc_connection.send(self.get_next_hop_if_name(call["destination_hash"]))
 
@@ -581,7 +831,14 @@ class Reticulum:
                     if path == "packet_snr":
                         rpc_connection.send(self.get_packet_snr(call["packet_hash"]))
 
+                if "drop" in call:
+                    path = call["drop"]
+
+                    if path == "path":
+                        rpc_connection.send(self.drop_path(call["destination_hash"]))
+
                 rpc_connection.close()
+
             except Exception as e:
                 RNS.log("An error ocurred while handling RPC call from local client: "+str(e), RNS.LOG_ERROR)
 
@@ -592,7 +849,7 @@ class Reticulum:
             response = rpc_connection.recv()
             return response
         else:
-            stats = []
+            interfaces = []
             for interface in RNS.Transport.interfaces:
                 ifstats = {}
                 
@@ -601,14 +858,86 @@ class Reticulum:
                 else:
                     ifstats["clients"] = None
 
+                if hasattr(interface, "b32"):
+                    if interface.b32 != None:
+                        ifstats["i2p_b32"] = interface.b32+".b32.i2p"
+                    else:
+                        ifstats["i2p_b32"] = None
+
+                if hasattr(interface, "bitrate"):
+                    if interface.bitrate != None:
+                        ifstats["bitrate"] = interface.bitrate
+                    else:
+                        ifstats["bitrate"] = None
+
+                if hasattr(interface, "peers"):
+                    if interface.peers != None:
+                        ifstats["peers"] = len(interface.peers)
+                    else:
+                        ifstats["peers"] = None
+
+                if hasattr(interface, "ifac_signature"):
+                    ifstats["ifac_signature"] = interface.ifac_signature
+                    ifstats["ifac_size"] = interface.ifac_size
+                    ifstats["ifac_netname"] = interface.ifac_netname
+                else:
+                    ifstats["ifac_signature"] = None
+                    ifstats["ifac_size"] = None
+                    ifstats["ifac_netname"] = None
+
+                if hasattr(interface, "announce_queue"):
+                    if interface.announce_queue != None:
+                        ifstats["announce_queue"] = len(interface.announce_queue)
+                    else:
+                        ifstats["announce_queue"] = None
+
                 ifstats["name"] = str(interface)
                 ifstats["rxb"] = interface.rxb
                 ifstats["txb"] = interface.txb
                 ifstats["status"] = interface.online
-                stats.append(ifstats)
+                ifstats["mode"] = interface.mode
+
+                interfaces.append(ifstats)
+
+            stats = {}
+            stats["interfaces"] = interfaces
+            if Reticulum.transport_enabled():
+                stats["transport_id"] = RNS.Transport.identity.hash
 
             return stats
 
+    def get_path_table(self):
+        if self.is_connected_to_shared_instance:
+            rpc_connection = multiprocessing.connection.Client(self.rpc_addr, authkey=self.rpc_key)
+            rpc_connection.send({"get": "path_table"})
+            response = rpc_connection.recv()
+            return response
+
+        else:
+            path_table = []
+            for dst_hash in RNS.Transport.destination_table:
+                entry = {
+                    "hash": dst_hash,
+                    "timestamp": RNS.Transport.destination_table[dst_hash][0],
+                    "via": RNS.Transport.destination_table[dst_hash][1],
+                    "hops": RNS.Transport.destination_table[dst_hash][2],
+                    "expires": RNS.Transport.destination_table[dst_hash][3],
+                    "interface": str(RNS.Transport.destination_table[dst_hash][5]),
+                }
+                path_table.append(entry)
+
+            return path_table
+
+    def drop_path(self, destination):
+        if self.is_connected_to_shared_instance:
+            rpc_connection = multiprocessing.connection.Client(self.rpc_addr, authkey=self.rpc_key)
+            rpc_connection.send({"drop": "path", "destination_hash": destination})
+            response = rpc_connection.recv()
+            return response
+
+        else:
+            return RNS.Transport.expire_path(destination)
+
     def get_next_hop_if_name(self, destination):
         if self.is_connected_to_shared_instance:
             rpc_connection = multiprocessing.connection.Client(self.rpc_addr, authkey=self.rpc_key)
@@ -686,12 +1015,18 @@ __default_rns_config__ = '''# This is the default Reticulum config file.
 # You should probably edit it to include any additional,
 # interfaces and settings you might need.
 
+# Only the most basic options are included in this default
+# configuration. To see a more verbose, and much longer,
+# configuration example, you can run the command:
+# rnsd --exampleconfig
+
+
 [reticulum]
 
 # If you enable Transport, your system will route traffic
 # for other peers, pass announces and serve path requests.
-# This should be done for systems that are suited to act
-# as transport nodes, ie. if they are stationary and
+# This should only be done for systems that are suited to
+# act as transport nodes, ie. if they are stationary and
 # always-on. This directive is optional and can be removed
 # for brevity.
 
@@ -719,6 +1054,7 @@ share_instance = Yes
 shared_instance_port = 37428
 instance_control_port = 37429
 
+
 # You can configure Reticulum to panic and forcibly close
 # if an unrecoverable interface error occurs, such as the
 # hardware device for an interface disappearing. This is
@@ -761,265 +1097,5 @@ loglevel = 4
   [[Default Interface]]
     type = AutoInterface
     interface_enabled = True
-    outgoing = True
-
-
-  # The following example enables communication with other
-  # local Reticulum peers using UDP broadcasts.
-  
-  [[UDP Interface]]
-    type = UDPInterface
-    interface_enabled = False
-    outgoing = True
-    listen_ip = 0.0.0.0
-    listen_port = 4242
-    forward_ip = 255.255.255.255
-    forward_port = 4242
-
-    # The above configuration will allow communication
-    # within the local broadcast domains of all local
-    # IP interfaces.
-
-    # Instead of specifying listen_ip, listen_port,
-    # forward_ip and forward_port, you can also bind
-    # to a specific network device like below.
-
-    # device = eth0
-    # port = 4242
-
-    # Assuming the eth0 device has the address
-    # 10.55.0.72/24, the above configuration would
-    # be equivalent to the following manual setup.
-    # Note that we are both listening and forwarding to
-    # the broadcast address of the network segments.
-
-    # listen_ip = 10.55.0.255
-    # listen_port = 4242
-    # forward_ip = 10.55.0.255
-    # forward_port = 4242
-
-    # You can of course also communicate only with
-    # a single IP address
-
-    # listen_ip = 10.55.0.15
-    # listen_port = 4242
-    # forward_ip = 10.55.0.16
-    # forward_port = 4242
-
-
-  # This example demonstrates a TCP server interface.
-  # It will listen for incoming connections on the
-  # specified IP address and port number.
-  
-  [[TCP Server Interface]]
-    type = TCPServerInterface
-    interface_enabled = False
-    outgoing = True
-
-    # This configuration will listen on all IP
-    # interfaces on port 4242
-    
-    listen_ip = 0.0.0.0
-    listen_port = 4242
-
-    # Alternatively you can bind to a specific IP
-    
-    # listen_ip = 10.0.0.88
-    # listen_port = 4242
-
-    # Or a specific network device
-    
-    # device = eth0
-    # port = 4242
-
-
-  # To connect to a TCP server interface, you would
-  # naturally use the TCP client interface. Here's
-  # an example. The target_host can either be an IP
-  # address or a hostname
-
-  [[TCP Client Interface]]
-    type = TCPClientInterface
-    interface_enabled = False
-    outgoing = True
-    target_host = 127.0.0.1
-    target_port = 4242
-
-
-  # Here's an example of how to add a LoRa interface
-  # using the RNode LoRa transceiver.
-
-  [[RNode LoRa Interface]]
-    type = RNodeInterface
-
-    # Enable interface if you want use it!
-    interface_enabled = False
-
-    # Allow transmit on interface. Setting
-    # this to false will create a listen-
-    # only interface.
-    outgoing = true
-
-    # Serial port for the device
-    port = /dev/ttyUSB0
-
-    # Set frequency to 867.2 MHz
-    frequency = 867200000
-
-    # Set LoRa bandwidth to 125 KHz
-    bandwidth = 125000
-
-    # Set TX power to 7 dBm (5 mW)
-    txpower = 7
-
-    # Select spreading factor 8. Valid 
-    # range is 7 through 12, with 7
-    # being the fastest and 12 having
-    # the longest range.
-    spreadingfactor = 8
-
-    # Select coding rate 5. Valid range
-    # is 5 throough 8, with 5 being the
-    # fastest, and 8 the longest range.
-    codingrate = 5
-
-    # You can configure the RNode to send
-    # out identification on the channel with
-    # a set interval by configuring the
-    # following two parameters. The trans-
-    # ceiver will only ID if the set
-    # interval has elapsed since it's last
-    # actual transmission. The interval is
-    # configured in seconds.
-    # This option is commented out and not
-    # used by default.
-    # id_callsign = MYCALL-0
-    # id_interval = 600
-
-    # For certain homebrew RNode interfaces
-    # with low amounts of RAM, using packet
-    # flow control can be useful. By default
-    # it is disabled.
-    flow_control = False
-    
-    
-  # An example KISS modem interface. Useful for running
-  # Reticulum over packet radio hardware.
-
-  [[Packet Radio KISS Interface]]
-    type = KISSInterface
-
-    # Enable interface if you want use it!
-    interface_enabled = False
-
-    # Allow transmit on interface.
-    outgoing = true
-
-    # Serial port for the device
-    port = /dev/ttyUSB1
-
-    # Set the serial baud-rate and other
-    # configuration parameters.
-    speed = 115200    
-    databits = 8
-    parity = none
-    stopbits = 1
-
-    # Set the modem preamble. A 150ms
-    # preamble should be a reasonable
-    # default, but may need to be
-    # increased for radios with slow-
-    # opening squelch and long TX/RX
-    # turnaround
-    preamble = 150
-
-    # Set the modem TX tail. In most
-    # cases this should be kept as low
-    # as possible to not waste airtime.
-    txtail = 10
-
-    # Configure CDMA parameters. These
-    # settings are reasonable defaults.
-    persistence = 200
-    slottime = 20
-
-    # You can configure the interface to send
-    # out identification on the channel with
-    # a set interval by configuring the
-    # following two parameters. The KISS
-    # interface will only ID if the set
-    # interval has elapsed since it's last
-    # actual transmission. The interval is
-    # configured in seconds.
-    # This option is commented out and not
-    # used by default.
-    # id_callsign = MYCALL-0
-    # id_interval = 600
-
-    # Whether to use KISS flow-control.
-    # This is useful for modems that have
-    # a small internal packet buffer, but
-    # support packet flow control instead.
-    flow_control = false
-
-
-  # If you're using Reticulum on amateur radio spectrum,
-  # you might want to use the AX.25 KISS interface. This
-  # way, Reticulum will automatically encapsulate it's
-  # traffic in AX.25 and also identify your stations
-  # transmissions with your callsign and SSID.
-  # 
-  # Only do this if you really need to! Reticulum doesn't
-  # need the AX.25 layer for anything, and it incurs extra
-  # overhead on every packet to encapsulate in AX.25.
-  #
-  # A more efficient way is to use the plain KISS interface
-  # with the beaconing functionality described above.
-
-  [[Packet Radio AX.25 KISS Interface]]
-    type = AX25KISSInterface
-
-    # Set the station callsign and SSID
-    callsign = NO1CLL
-    ssid = 0
-
-    # Enable interface if you want use it!
-    interface_enabled = False
-
-    # Allow transmit on interface.
-    outgoing = true
-
-    # Serial port for the device
-    port = /dev/ttyUSB2
-
-    # Set the serial baud-rate and other
-    # configuration parameters.
-    speed = 115200    
-    databits = 8
-    parity = none
-    stopbits = 1
-
-    # Whether to use KISS flow-control.
-    # This is useful for modems with a
-    # small internal packet buffer.
-    flow_control = false
-
-    # Set the modem preamble. A 150ms
-    # preamble should be a reasonable
-    # default, but may need to be
-    # increased for radios with slow-
-    # opening squelch and long TX/RX
-    # turnaround
-    preamble = 150
-
-    # Set the modem TX tail. In most
-    # cases this should be kept as low
-    # as possible to not waste airtime.
-    txtail = 10
-
-    # Configure CDMA parameters. These
-    # settings are reasonable defaults.
-    persistence = 200
-    slottime = 20
 
 '''.splitlines()
